/* 
 *  @TITLE "test1.c"
 *
 *  This file contains a test driver for the modularized diskdevice
 *  module.  In this test, main() reads disk requests from the given file,
 *  executing them by calling the disk model. 
 *
 * Each input line should look like a space-separated set of 4 numbers:
 *  disk    write   sector   nsectors
 * where disk is the disk number (always 1 here),
 * where write is 1 if write request, 0 if read request
 * where sector is the starting sector number in the logical sector 
 *    address space of the disk device
 * where nsectors is the number of sectors to transfer.
 *
 * For each input line, a line is output giving the length of the transfer,
 * the time (in usec) for the transfer, and a counter.
 * 
 * #ifdef REAL_DATA, 
 *   The program uses a file called 'vdisk' as its virtual disk.
 *
 *     Song Bac Toh
 *     July 1994
 */
/* $Id: test1.c,v 2.0 94/07/17 18:35:24 dfk Exp $ */


#include "diskdevice.h"
#include "modularize.h"
#include "queue.h"
#define BQSIZE    15		/* size of bus queue */
#define MSECperUSEC     ( 1.0 / 1000.0 )
#define TICSperUSEC     (TICSperMSEC * MSECperUSEC)
#define STUFFY          "I am a stuffed string"
#define DISKFILE        "vdisk"

extern void srandom(int seed);
Heap heap;			/* the event heap */


void main(int argc, char **argv)
{
  ulong disk;
  ulong sector, nsectors;
  boolean write;
  char *buffer;
  char busString[] = "bus queue";
  NCQUEUE *busWaitq;
  int busOwner;
  int busId;
  int counter, lengthOfString;
  TICS begintime, endtime, dtime;
  ulong nSectors, sectorSize, nTracks, sectorsPerTrack;
  ulong beginsector, endsector, dsectors;
  FILE *inputfile;
  char inputbuf[BUFSIZ+1];
  int i,n;

  /* ------------ can only take 1 paramter ---------- */
  if (argc != 2)
    {
      fprintf(stderr, "syntax: %s testinput\n", argv[0]);
      exit(FATAL);
    }
  /* ------------------------------------------------ */

  if ((inputfile = fopen(argv[1], "r")) == NULL) {
      fprintf(stderr, "%s: file '%s' cannot be read\n", argv[0], argv[1]);
      exit(FATAL);
  }

  srandom(time((time_t *)0));	    /* plant a random seed */
  InitTime();		            /* initialize the global clock */

  /* In this test driver, I only use 1 disk, ie disk 1 */
  busOwner = BUS_FREE;
  busWaitq = MakeQueue_noncyc(BQSIZE, busString);

  /* Each disk has a bus; but since I only use disk 1, I only initialize
     a disk and its bus */
  disk = 1;
  busId = 1;

#ifdef REAL_DATA		/* want to write real data to disk */
  DiskDeviceInit(disk, busId, &busOwner, busWaitq, DISKFILE);
#else
  DiskDeviceInit(disk, busId, &busOwner, busWaitq, NULL);
#endif

  DiskDeviceShape(disk, &nSectors, &sectorSize, &nTracks, &sectorsPerTrack);

#ifdef REAL_DATA		/* want to write real data to disk */
  /* make room for one sector now; reallocate later as needed */
  buffer = (char *) malloc(sectorSize);
  if (buffer == NULL) {
      fprintf(stderr, "can't allocate memory for buffer\n");
      exit(FATAL);
  }
#else
  buffer = NULL;
#endif

  counter = 0;

  /* Read the source trace file till the end of file */
  while(fgets(inputbuf, BUFSIZ, inputfile) != NULL) {
      counter++;	      /* increment the counter */

      n = sscanf(inputbuf, "%lu%d%lu%lu", &disk, &write, &sector, &nsectors);

      if (n != 4) {
	  fprintf(stderr, "%s: INVALID input line, file %s, line %d: '%s' (found %d numbers)\n",
		  argv[0], argv[1], counter, inputbuf, n);
	  continue;
      }

      begintime = GetTime();  /* start time */

#ifdef REAL_DATA	      /* want to write real data to disk */
      buffer = (char *) realloc(buffer, sectorSize * nsectors);
      if (buffer == NULL) {
	  fprintf(stderr, "can't allocate memory for buffer\n");
	  exit(FATAL);
      }

      if (write) {
	  lengthOfString = strlen(STUFFY);
	  buffer[0] = '\0';
	  for (i = 1; (lengthOfString * i < sectorSize*nsectors); i++)
	    strcat(buffer, STUFFY);
      }
#else
      buffer = NULL;
#endif


      /* call either DiskDeviceTransfer() or DiskDeviceStart/Finish() */
      /* DEBUG1 turns the statement on, DEBUG0 turns it off           */
      DEBUG1 DiskDeviceTransfer(disk, sector, nsectors, write, buffer);

      DEBUG0 DiskDeviceStart(disk, sector, nsectors, write);
      DEBUG0 DiskDeviceFinish(disk, buffer);

      /* Synchronize the disk with its buffer.  Doesn't do anything if
	 we are not using immediately report write */
      DEBUG1 DiskDeviceSync(disk);

      endtime = GetTime();    /* end time */

      dsectors = endsector - beginsector;
      dtime = endtime - begintime;

      free(buffer);

      /* The following are some debugging info that can be turned
	 on or off */
      DEBUG0 printf("begin time = %s\t", time_print(begintime));
      DEBUG0 printf("end time = %s\n", time_print(endtime));
      DEBUG0 printf("%20d\t%10g\n", 
		    abs(dsectors), (dtime / TICSperMSEC));

      /* print the time in milliseconds */
      DEBUG0 printf("%20lu\t%s\n", 
		    nsectors, time_print(dtime/TICSperUSEC));

      /* this statement prints it with a counter showing how many
	 requests has been done */
      DEBUG1 printf("%20lu\t%s\t%d\n", 
		    nsectors, time_print(dtime/TICSperUSEC), counter);
  }
  
  /* Done with the disk, reset the info and shut it down */
  DiskDeviceDone(disk);  
}
